package LATMOS::Accounts::Web::Model::AttrForms;

use strict;
use warnings;
use base 'Catalyst::Model';

=head1 NAME

LATMOS::Accounts::Web::Model::AttrForms - Catalyst Model

=head1 DESCRIPTION

Catalyst Model.

=cut

my $attrs = {
    uid => [ 'Login', 'text:12' ],
    uidNumber => [ 'UID', 'text-U:6' ],
    gidNumber => [ 'GID', 'select:group:gidNumber' ],
    manager => [ 'Responsable', 'select-N:user' ],
    managedBy => [ 'Responsable', 'select-N:user' ],
    sn => [ 'Nom' ],
    givenName => [ 'Prénom' ],
    homeDirectory => [ 'Home', 'text-U:25' ],
    loginShell => [ 'Shell', 'text-A' ],
    physicalDeliveryOfficeName => [ 'Bureau' ],
    telephoneNumber => [ 'Téléphone' ],
    otherTelephone => [ 'Téléphone (autre)' ],
    company => [ 'Société', 'text-A' ],
    l => [ 'Ville' ],
    postalCode => [ 'Code postal' ],
    postOfficeBox => [ 'BP' ],
    department => [ 'Département', 'select-N:group/sutype=dpmt' ],
    streetAddress => [ 'Rue', 'textarea' ],
    title => [ 'Fonction' ],
    expire => [ 'Expire le', 'date' ],
    st => [ 'État (US)' ],
    sutype => [ 'Structure admin.', 'select-N:sutype' ],
    exported => [ 'Propagé', 'label' ],
    locked => [ 'Vérouillé', 'checkbox:l' ],
    isMainAddress => [ 'Adresse principale', 'checkbox' ],
    site => [ 'Site', 'select-N:site' ],
    co => [ 'Pays' ],
    mail => [ 'Mail', 'text-U:30' ],
    grade => [ 'Grade', 'text-A' ],
    facsimileTelephoneNumber => [ 'Fax', 'text' ],
    nickname => [ 'Nick', 'text-U:3' ],
    cn => [ 'Nom' ],
    initials => [ 'Initiales', 'text:5' ],
    create => [ 'Créé le' ],
    date => [ 'Modifié le' ],
    college => [ 'College', 'select-N:group/sutype=college' ],
    contratType => [ 'Type de contrat', 'select-N:group/sutype=contrattype' ],
    managerContact => [ 'Contact' ],
};

my $forms = {
    user => {
        name => 'Système',
        attrs => [ qw(
            sn givenName description
            uid uidNumber gidNumber gecos homeDirectory loginShell
            mail initials nickname
            locked
            exported
            create
            date
        ) ],
    },
    useraddress => {
        name => 'Adresse',
        attrs => [ qw(
            initials
            mail
        ) ],
    },
    userstatus => {
        name => 'Status',
        attrs => [ qw(
            company
            department
            manager
            managerContact
            contratType
            expire
        ) ],
    },
    site => {
        name => 'Site',
        attrs => [ qw(
            description
            streetAddress
            postOfficeBox
            postalCode
            l
            st
            co
            facsimileTelephoneNumber
            create
            date
        ) ],
    },
    address => {
        name => 'Adresse',
        attrs => [ qw(
            isMainAddress
            telephoneNumber
            streetAddress
            postOfficeBox
            postalCode
            l
            st
            physicalDeliveryOfficeName
            site
            co
            exported
            description
        ) ],
    },
    group => {
        name => 'Système',
        attrs => [ qw(
            gidNumber description
            managedBy
            sutype exported
            create
            date
        ) ],
    },
};

sub escape {
    my ($self, $text) = @_;
    $text ||= '';
    for ($text) {
        s/&/&amp;/g;
        s/</&lt;/g;
        s/>/&gt;/g;
        s/"/&quot;/g;
    }
    $text;
}

sub new {
    my ($class) = @_;
    bless({}, $class);
}

# call either OBJ or type + base

sub ACCEPT_CONTEXT {
    my ($self, $c, $form, $object, $base) = @_;
    my $new = {};
    $new->{c} = $c;
    $new->{form} = $form;
    $new->{object} = $object if (ref $object);
    $new->{base} = $base || ($object ? $object->base : undef) or return $self;
    $new->{otype} = ref $object ? $object->type : $object;
    bless($new, 'LATMOS::Accounts::Web::Model::AttrForms');
}

sub base {
    my ( $self ) = @_;
    $self->{base}
}

sub otype {
    my ($self) = @_;
    $self->{otype};
}

sub label {
    my ($self) = @_;
    $forms->{$self->{form}}->{name} || ''
}

sub attributes {
    my ($self, $for) = @_;
    grep { $_ }
    grep { $self->base->check_acl($self->{object} || $self->otype, $_, 'r') }
    grep { $self->base->get_field_name($self->otype, $_, $for || 'a') }
    @{ $forms->{$self->{form}}->{attrs} };
}

sub attr_label {
    my ($self, $attr) = @_;
    my $htmlname = ($self->{object} ? $self->{object}->id . '_' : '') . $attr;
    return sprintf('<label for="%s">%s</label>',
        $self->escape($htmlname),
        $self->escape($attrs->{$attr}[0] || $attr)
    );
}

sub attr_raw_value {
    my ($self, $attr) = @_;
    return $self->{c}->req->param($attr) ||
        ($self->{object} ? $self->{object}->get_c_field($attr) : '')
}

sub attr_field {
    my ($self, $attr, $type) = @_;
    my $modallow = $self->base->check_acl($self->{object}
        ? ($self->{object}, $attr, 'w')
        : ($self->otype, '@CREATE', 'w'));
    $type ||= $attrs->{$attr}[1] || '';
    if (!($self->base->get_field_name($self->otype, $attr, 'w') && $modallow)) {
        $type = 'label';
    }
    # exception: gidNumber is used also in group, but we don't want
    # group list here, really the number !
    $type = $modallow ? 'text-U:6' : 'label' if ($self->{otype} eq 'group' && $attr eq 'gidNumber');
    $type ||= 'text';
    
    my $htmlname = $self->escape(($self->{object} 
            ? $self->{object}->id . '_'
            : ''
        ) . $attr
    );
    for ($type) {
        /^textarea$/ and return sprintf(
            '<textarea id="%s" name="%s" cols="40">%s</textarea>',
            $self->escape($htmlname),
            $self->escape($htmlname),
            $self->escape($self->attr_raw_value($attr) || ''),
        );
        /^label$/ and do {
            my $field = $self->escape(
                $self->attr_raw_value($attr)
            );
            $field =~ s/\n/<br>/g;
            return $field . sprintf('<input type="hidden" name="%s" value="%s">',
                $self->escape($htmlname), ($self->attr_raw_value($attr) || ''));
        };
        /^date$/ and do {
            my ($date, $time) = split(/ /, $self->attr_raw_value($attr) || '');
            if ($date && $date =~ /^(\d+)-(\d+)-(\d+)$/) {
                $date = "$3/$2/$1";
            }
            my $html = "\n" . q{<SCRIPT LANGUAGE="JavaScript" ID="js13">
            var cal13 = new CalendarPopup();
            </SCRIPT>} . "\n";
            $html .= sprintf(
                '<input type="text" id="%s" name="%s" value="%s" size="12">',
                $self->escape($htmlname),
                $self->escape($htmlname),
                $self->escape($date)
            );
            $html .= q{<DIV ID="testdiv1" STYLE="position:absolute;visibility:hidden;background-color:white;layer-background-color:white;"></DIV>};
            $html .= qq|
            <A HREF="#"
                onClick="cal13.select(document.forms[0].$htmlname,'${htmlname}_anc','dd/MM/yyyy');return false;"
                TITLE="cal13.select(document.forms[0].$htmlname,'${htmlname}_anc','dd/MM/yyyy');return false;"
                NAME="${htmlname}_anc" ID="${htmlname}_anc">
                <img src="| . $self->{c}->uri_for(qw(/static icons view-calendar-day.png))
                . qq{" style="ref"></A>
                } . "\n";
            return $html;
        };
        /^checkbox(?::(\w+))?$/ and do {
            my $options = $1 || '';
            my $text = sprintf('<input type="checkbox" name="%s"%s>',
                $self->escape($htmlname),
                $self->attr_raw_value($attr) ? '  checked="yes"' : ''
            ); 
            $text .= sprintf('<input type="hidden" name="%s">',
                $self->escape($htmlname));
            if ($options =~ /l/) {
                $text .= $self->attr_raw_value($attr)
                    ? ' ' . $self->attr_raw_value($attr)
                    : ''; 
            }
            return $text;
        };
        /^select(-\w+)?:([^:\/]+)(?:\/([^:]+))?(?::(.*))?/ and do {
            my $options = $1 || '';
            my $otype = $2;
            my $filter = $3;
            my $keyfield = $4;
            my $observe_keyfield = $keyfield || 'displayName';
            my $select = sprintf('<select id="%s" name="%s">',
                $self->escape($htmlname),
                $self->escape($htmlname)) . "\n";
            $select .= '<option value="">--</option>' . "\n" if ($options =~ /N/);
            my $value = $self->attr_raw_value($attr) || '';
            my $initial_observed = '';
            foreach my $id (sort $filter
                ? $self->base->search_objects($otype, $filter)
                : $self->base->list_objects($otype)) {
                my $val = $id;
                if ($keyfield) {
                    my $obj = $self->base->get_object($otype, $id) or next;
                    $val = $obj->get_c_field($keyfield);
                }
                $select .= sprintf(
                    '    <option value="%s"%s>%s</option>',
                    $self->escape($val || ''),
                    $value eq $val ? ' selected="selected"' : '',
                    $self->escape($id || ''),
                );
                $select .= "\n";
                if($value eq $val) {
                    if (my $obj = $self->base->get_object($otype, $id)) {
                        $initial_observed = $obj->get_c_field($observe_keyfield)
                        || '';
                    }
                }
            }
            $select .= "</select>\n";
            $select .= $self->{c}->prototype->observe_field( $htmlname, {
                update => "${htmlname}_span",
                url => $self->{c}->uri_for('/ajax', 'rawattr', $otype),
                frequency => 1,
                with   => "'attr=" . $observe_keyfield .
                "&id='+element.options[element.selectedIndex].text",
            }) .
            qq|<span id="${htmlname}_span">$initial_observed</span>|;
            return $select;
        };
        /^text(-\w+)?(?::(\d+))?/ and do {
            my $flag = $1 || '';
            if (my @allowed = $self->base->obj_attr_allowed_values(
                    $self->{otype}, $attr)) {
                my $cvalue = $self->attr_raw_value($attr);
                my $textf = sprintf('<select  id="%s" name="%s">',
                    $self->escape($htmlname),
                    $self->escape($htmlname)) . "\n";
                $textf .= '<option value="">--</option>' . "\n";
                foreach (sort @allowed) {
                    $textf .= sprintf('<option value="%s"%s>%s</option>' . "\n",
                        $self->escape($_),
                        (($cvalue || '') eq $_ ? ' selected="selected"' : ''),
                        $self->escape($_),
                    );
                }
                $textf .= "</select>\n";
                return $textf;
            } else {
                my $textf = sprintf(
                    '<input type="text" id="%s" name="%s" value="%s" size="%d">',
                    $self->escape($htmlname),
                    $self->escape($htmlname),
                    $self->escape($self->attr_raw_value($attr)),
                    $2 || 30,
                );
                if ($flag =~ /A/) {
                $textf .= qq|<span id="${htmlname}_auto_complete"></span>|;
                $textf .= "\n";
                $textf .= $self->{c}->prototype->auto_complete_field(
                    $htmlname,
                    {
                    update => "${htmlname}_auto_complete",
                    url => $self->{c}->uri_for('/ajax', 'attrvalues', $self->otype, $attr),
                    indicator => "${htmlname}_stat", min_chars => 1,
                    with => "'val='+document.getElementById('$htmlname').value",
                    frequency => 2,
                    }
                );
                }
                if ($flag =~ /U/) {
                $textf .= qq|<span id="${htmlname}_observer_uniq"></span>|;
                $textf .= "\n";
                $textf .= $self->{c}->prototype->observe_field(
                    $htmlname,
                    {
                    update => "${htmlname}_observer_uniq",
                    url => $self->{c}->uri_for('/ajax', 'objattrexist',
                        $self->otype, $attr),
                    frequency => 2,
                    indicator => "${htmlname}_stat", min_chars => 1,
                    with => "'val='+document.getElementById('$htmlname').value" .
                        ($self->{object} ? "+'&exclude=" . $self->{object}->id . "'" :
                            ''),
                    }
                );
                }
                $textf .= qq|<span style="display:none" id="${htmlname}_stat">Searching...</span>|;

                return $textf;
            }
        };
    }
}

sub submit {
    my ($self) = @_;
    return sprintf(
        '<input type="submit" name="%s" value="Enregistrer">',
        $self->escape($self->label),
    );
}

sub set_attrs {
    my ($self) = @_;
    $self->{c}->req->param($self->label) or return;
    my $prefix = $self->{object}->id . '_';
    my %fields;
    foreach (
        grep { $self->base->get_field_name($self->otype, $_, 'w') }
        $self->attributes) {
        if (($attrs->{$_}[1] || '') eq 'checkbox') {
            $fields{$_} = $self->{c}->req->param("$prefix$_") ? 1 : 0;
        } else {
            $fields{$_} = $self->{c}->req->param("$prefix$_");
        }
    }
    $self->{object}->set_c_fields(%fields) or return;
    $self->{object}->base->commit;
}

=head1 AUTHOR

Thauvin Olivier

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
